//**Do not*** #include <TimeLib.h> by Paul Stoffregen, we will mimic it!
// https://www.esp8266.com/viewtopic.php?f=29&t=21327

#include <NTPClient.h>    // NTP Client by Fabrice Weinberg
#if defined(ESP8266)
#include <ESP8266WiFi.h>  // default from Espressif
#include <WiFiUdp.h>      // default from Espressif 
#elif defined(ESP32)
#include <WiFi.h>         // default from Espressif
#endif
#include "secrets.h"

#define HOST_NAME "ESP-IoT"
#define TZ            1                  // (utc+) TZ in hours
#define DST_MN        60                 // use 60mn for summer time in some countries
#define GMT_OFFSET_SEC 3600 * TZ         // Do not change here...
#define DAIYLIGHT_OFFSET_SEC 60 * DST_MN // Do not change here...
#define NTP_SERVER "hu.pool.ntp.org"

// Variables for Time
static int lastSecond;
tm*        timeinfo;                    //localtime returns a pointer to a tm structstatic int Second;
time_t     Epoch;

static int Second;
static int Minute;
static int Hour;
static int Day;
static int Month;
static int Year;
static int Weekday;
char       DayName[12];
char       MonthName[12];
char       Time[10];
char       Date[12];

static IPAddress ip;

//*** Buffers ***
static char charbuff[80];    //Char buffer for many functions
//String text;               //String buffer for many functions

// Constructors
WiFiUDP ntpUDP;
NTPClient timeClient(ntpUDP, NTP_SERVER, GMT_OFFSET_SEC);

void getWiFi() {
  WiFi.begin(WIFI_SSID, WIFI_PASS);
  while ( WiFi.status() != WL_CONNECTED ) {
    delay ( 500 );
    Serial.print ( "." );
  }
  ip = WiFi.localIP();
  sprintf(charbuff, "Connected to IP: %03d.%03d.%03d.%03d", ip[0], ip[1], ip[2], ip[3]);
  Serial.println(charbuff);
}

void getNTP() {
  timeClient.begin();
  timeClient.update();
  Serial.print(timeClient.getFormattedTime());
}
void getEpoch() {
  Epoch   = timeClient.getEpochTime();
}
void getTimeData() {
  timeinfo  = localtime(&Epoch);  // cf: https://www.cplusplus.com/reference/ctime/localtime/
  Second    = timeinfo->tm_sec;
  Minute    = timeinfo->tm_min;
  Hour      = timeinfo->tm_hour;
  Weekday   = timeinfo->tm_wday + 1 ;
  Day       = timeinfo->tm_mday;
  Month     = timeinfo->tm_mon + 1;
  Year      = timeinfo->tm_year + 1900; //returns years since 1900
  strftime (DayName , 12, "%A", timeinfo); //cf: https://www.cplusplus.com/reference/ctime/strftime/
  strftime (MonthName, 12, "%B", timeinfo);
  strftime (Time, 10, "%T", timeinfo);
  strftime (Date, 12, "%d/%m/%Y", timeinfo);
}

void setup() {
  Serial.begin(115200);
  getWiFi();
  getNTP();

}

void loop() {
  timeClient.update();   // should be called from time to time to refresh NTP when expired, does nothing most of the time.
  getEpoch();            // writes the Epoch (Numbers of seconds till 1.1.1970...
  getTimeData();         // breaks down the Epoch into discrete values.

  // Examples with Strings
  Serial.print(F("Thanks God, it is: "));   Serial.print( DayName );
  Serial.print(F("! Date is "));   Serial.print( Date );
  Serial.print(F(" and Time is "));   Serial.println( Time );

  // Examples with a character buffer (can be used for displays as well)
  sprintf(charbuff, "Now is %02d Hour, %02d Minutes and %02d Seconds. The Epoch is: %10lu" , Hour , Minute, Second, Epoch);
  Serial.println(charbuff);
  sprintf(charbuff, "Date is %s, %02d %s %04d ", DayName , Day , MonthName, Year);
  Serial.print(charbuff);
  Serial.println();

  // Examples with Posix expressions
  strftime (charbuff, 80, "Americans say it is %I:%M%p, but Europeans prefer using %T", timeinfo);
  Serial.println(charbuff);
  Serial.println();
  Serial.println();
  delay(1000);
}
