/* ESP82366_ntp_clock
 *  
 * NTP óra I2C OLED kijelzővel, amely hálózatról kéri le 
 * a pontos időt. Az alábbi mintapélda 128x32 felbontású 
 * SSD1306 vezérlőjű kijelzőhöz van konfigurálva, de könnyen 
 * módosítható 128x64 felbontású SSD1306 vagy SH1106 kijelzőhöz.
 * 
 * Tested with Arduino IDE 1.8.13 and ESP8266 Arduino core 2.7.4
 */

#include <NTPClient.h>    // NTP Client by Fabrice Weinberg
#include <ESP8266WiFi.h>  // default from Espressif
#include <WiFiUdp.h>      // default from Espressif 
#include "secrets.h"      // WiFi login secrets
#include <oled.h>

#define TZ            1                  // (utc+) időzóna órákban
#define DST_MN        60                 // nyári időszámítás 60 perc eltolás
#define GMT_OFFSET_SEC 3600 * TZ
#define DAIYLIGHT_OFFSET_SEC 60 * DST_MN
#define NTP_SERVER "hu.pool.ntp.org"     // magyarországi időszerver klaszter

// Variables 
tm*        timeinfo;      //localtime returns a pointer to a tm structure
time_t     Epoch;
static char msg[20];      //Char buffer 

// Constructors
WiFiUDP ntpUDP;
NTPClient timeClient(ntpUDP, NTP_SERVER, GMT_OFFSET_SEC);
OLED display=OLED(0,2,NO_RESET_PIN,0x3C,128,32,false);

void setup() {
  display.begin();
  WiFi.begin(WIFI_SSID, WIFI_PASS);
  while ( WiFi.status() != WL_CONNECTED ) {
    delay ( 500 );
  }
  timeClient.begin();
  timeClient.update();
}

void loop() {
  timeClient.update();
  Epoch   = timeClient.getEpochTime();
  timeinfo  = localtime(&Epoch);
  display.clear();
  strftime (msg, 15, "%Y-%b-%d %a ", timeinfo);
  display.draw_string(16,1,msg);
  strftime (msg, 10, "%T ", timeinfo);
  display.draw_string(8,12,msg,OLED::DOUBLE_SIZE);
  display.display();
  delay(1000);
}
